@@ -63,8 +63,10 @@ class Run(Annotate):  # noqa: D101
                                           output files. [default: .]
             --antismash-sideload          write an AntiSMASH v6 sideload JSON
                                           file next to the output files.
-            --force-clusters-tsv          always write a ``clusters.tsv`` file
-                                          even when no clusters were found.
+            --force-tsv                   always write TSV output files even
+                                          when they are empty (e.g. because 
+                                          no genes or no clusters were found).
+                                          
 
         Parameters - Gene Calling:
             -M, --mask                    Enable unknown region masking to
@@ -84,7 +86,7 @@ class Run(Annotate):  # noqa: D101
                                           valid cluster must contain. [default: 3]
             -m <m>, --threshold <m>       the probability threshold for cluster
                                           detection. Default depends on the
-                                          post-processing method (0.3 for gecco,
+                                          post-processing method (0.8 for gecco,
                                           0.6 for antismash).
             --postproc <method>           the method to use for cluster validation
                                           (antismash or gecco). [default: gecco]
@@ -94,7 +96,7 @@ class Run(Annotate):  # noqa: D101
                                           included if they are longer. A lower
                                           number will increase the number of
                                           false positives on small contigs.
-                                          [default: 10]
+                                          [default: 0]
 
         Parameters - Debug:
             --model <directory>           the path to an alternative CRF model
@@ -122,44 +124,25 @@ class Run(Annotate):  # noqa: D101
                 optional=True,
             )
             if self.args["--threshold"] is None:
-                self.threshold = 0.3 if self.args["--postproc"] == "gecco" else 0.6
+                self.threshold = 0.8 if self.args["--postproc"] == "gecco" else 0.6
             else:
                 self.threshold = self._check_flag("--threshold", float, lambda x: 0 <= x <= 1, hint="number between 0 and 1")
             self.jobs = self._check_flag("--jobs", int, lambda x: x >= 0, hint="positive or null integer")
             self.postproc = self._check_flag("--postproc", str, lambda x: x in ("gecco", "antismash"), hint="'gecco' or 'antismash'")
             self.edge_distance = self._check_flag("--edge-distance", int, lambda x: x >= 0, hint="positive or null integer")
-            self.format = self._check_flag("--format")
+            self.format = self._check_flag("--format", optional=True)
             self.genome = self._check_flag("--genome")
-            self.model = self._check_flag("--model")
+            self.model = self._check_flag("--model", optional=True)
             self.hmm = self._check_flag("--hmm")
             self.output_dir = self._check_flag("--output-dir")
             self.antismash_sideload = self._check_flag("--antismash-sideload", bool)
-            self.force_clusters_tsv = self._check_flag("--force-clusters-tsv", bool)
+            self.force_tsv = self._check_flag("--force-tsv", bool)
             self.mask = self._check_flag("--mask", bool)
         except InvalidArgument:
             raise CommandExit(1)
 
     # ---
 
-    def _make_output_directory(self) -> None:
-        # Make output directory
-        self.info("Using", "output folder", repr(self.output_dir), level=1)
-        try:
-            os.makedirs(self.output_dir, exist_ok=True)
-        except OSError as err:
-            self.error("Could not create output directory: {}", err)
-            raise CommandExit(err.errno) from err
-
-        # Check if output files already exist
-        base, _ = os.path.splitext(os.path.basename(self.genome))
-        output_exts = ["features.tsv", "clusters.tsv"]
-        if self.antismash_sideload:
-            output_exts.append("sideload.json")
-        for ext in output_exts:
-            if os.path.isfile(os.path.join(self.output_dir, f"{base}.{ext}")):
-                self.warn("Output folder contains files that will be overwritten")
-                break
-
     def _load_model_domains(self) -> typing.Set[str]:
         try:
             if self.model is None:
@@ -195,15 +178,6 @@ class Run(Annotate):  # noqa: D101
             cpus=self.jobs
         ))
 
-    def _write_feature_table(self, genes):
-        from ...model import FeatureTable
-
-        base, _ = os.path.splitext(os.path.basename(self.genome))
-        pred_out = os.path.join(self.output_dir, f"{base}.features.tsv")
-        self.info("Writing", "feature table to", repr(pred_out), level=1)
-        with open(pred_out, "w") as f:
-            FeatureTable.from_genes(genes).dump(f)
-
     def _extract_clusters(self, genes):
         from ...refine import ClusterRefiner
 
@@ -281,6 +255,8 @@ class Run(Annotate):  # noqa: D101
                     "e-filter": repr(self.e_filter),
                     "postproc": repr(self.postproc),
                     "threshold": repr(self.threshold),
+                    "mask": repr(self.mask),
+                    "edge-distance": repr(self.edge_distance),
                 }
             }
         }
@@ -324,14 +300,22 @@ class Run(Annotate):  # noqa: D101
             self._check()
             ctx.enter_context(self.progress)
             ctx.enter_context(patch_showwarnings(self._showwarnings))
-            # attempt to create the output directory
-            self._make_output_directory()
+            # attempt to create the output directory, checking it doesn't
+            # already contain output files (or raise a warning)
+            extensions = ["features.tsv", "genes.tsv", "clusters.tsv"]
+            if self.antismash_sideload:
+                extensions.append("sideload.json")
+            self._make_output_directory(extensions)
             # load sequences and extract genes
             sequences = self._load_sequences()
             genes = self._extract_genes(sequences)
             if genes:
                 self.success("Found", "a total of", len(genes), "genes", level=1)
             else:
+                if self.force_tsv:
+                    self._write_genes_table(genes)
+                    self._write_feature_table([])
+                    self._write_cluster_table([])
                 self.warn("No genes were found")
                 return 0
             # use a whitelist for domain annotation, so that we only annotate
@@ -340,6 +324,7 @@ class Run(Annotate):  # noqa: D101
             # annotate domains and predict probabilities
             genes = self._annotate_domains(genes, whitelist=whitelist)
             genes = self._predict_probabilities(genes)
+            self._write_genes_table(genes)
             self._write_feature_table(genes)
             # extract clusters from probability vector
             clusters = self._extract_clusters(genes)
@@ -347,7 +332,7 @@ class Run(Annotate):  # noqa: D101
                 self.success("Found", len(clusters), "potential gene clusters", level=1)
             else:
                 self.warn("No gene clusters were found")
-                if self.force_clusters_tsv:
+                if self.force_tsv:
                     self._write_cluster_table(clusters)
                 return 0
             # predict types for putative clusters
