@@ -13,7 +13,7 @@ import pickle
 import random
 import signal
 import typing
-from typing import Any, Dict, Union, Optional, List, TextIO, Mapping
+from typing import Any, Dict, Union, Optional, List, Iterator, TextIO, Mapping
 
 from .._utils import in_context, patch_showwarnings, ProgressReader
 from ._base import Command, CommandExit, InvalidArgument
@@ -34,7 +34,7 @@ class Train(Command):  # noqa: D101
         gecco train - {cls.summary}
 
         Usage:
-            gecco train --features <table>... --clusters <table> [options]
+            gecco train --features <table>... [options]
 
         Arguments:
             -f <data>, --features <table>   a domain annotation table, used to
@@ -42,14 +42,19 @@ class Train(Command):  # noqa: D101
             -c <data>, --clusters <table>   a cluster annotation table, used to
                                             extract the domain composition for
                                             the type classifier.
+            -g <file>, --genes <file>       a gene table containing the
+                                            coordinates of the genes inside
+                                            the training sequence.
 
         Parameters:
-            -o <out>, --output-dir <out>    the directory to use for the model
-                                            files. [default: model]
             -j <jobs>, --jobs <jobs>        the number of CPUs to use for
                                             multithreading. Use 0 to use all
                                             the available CPUs. [default: 0]
 
+        Parameters - Output:
+            -o <out>, --output-dir <out>    the directory to use for the model
+                                            files. [default: model]
+
         Parameters - Domain Annotation:
             -e <e>, --e-filter <e>          the e-value cutoff for domains to
                                             be included.
@@ -64,15 +69,17 @@ class Train(Command):  # noqa: D101
                                             [default: 42]
 
         Parameters - Training:
+            -W <N>, --window-size <N>       the size of the sliding window for
+                                            CRF predictions. [default: 5]
+            --window-step <N>               the step of the sliding window for
+                                            CRF predictions. [default: 1]
             --c1 <C1>                       parameter for L1 regularisation.
                                             [default: 0.15]
             --c2 <C2>                       parameter for L2 regularisation.
                                             [default: 0.15]
-            --feature-type <type>           how features should be extracted
-                                            (single, overlap, or group).
-                                            [default: group]
-            --overlap <N>                   how much overlap to consider if
-                                            features overlap. [default: 2]
+            --feature-type <type>           at which level features should be
+                                            extracted (protein or domain).
+                                            [default: protein]
             --select <N>                    fraction of most significant features
                                             to select from the training data.
             --correction <method>           the multiple test correction method
@@ -89,14 +96,10 @@ class Train(Command):  # noqa: D101
             self.feature_type = self._check_flag(
                 "--feature-type",
                 str,
-                lambda x: x in {"single", "overlap", "group"},
-                hint="'single', 'overlap' or 'group'"
-            )
-            self.overlap = self._check_flag(
-                "--overlap",
-                int,
-                lambda x: x > 0,
-                hint="positive integer",
+                lambda x: x in {"protein", "domain"},
+                hint="'protein' or 'domain'",
+                default="protein",
+                optional=True,
             )
             self.c1 = self._check_flag("--c1", float, hint="real number")
             self.c2 = self._check_flag("--c2", float, hint="real number")
@@ -139,6 +142,19 @@ class Train(Command):  # noqa: D101
             self.output_dir = self._check_flag("--output-dir", str)
             self.features = self._check_flag("--features", list)
             self.clusters = self._check_flag("--clusters", str)
+            self.genes = self._check_flag("--genes", str)
+            self.window_size = self._check_flag(
+                "--window-size",
+                int,
+                lambda x: x > 0,
+                hint="positive integer",
+            )
+            self.window_step = self._check_flag(
+                "--window-step",
+                int,
+                lambda x: x > 0 and x <= self.window_size,
+                hint="positive integer smaller than `--window-size`",
+            )
         except InvalidArgument:
             raise CommandExit(1)
 
@@ -169,6 +185,22 @@ class Train(Command):  # noqa: D101
                 self.warn("Output folder contains files that will be overwritten")
                 break
 
+    def _load_genes(self) -> Iterator["Gene"]:
+        from ...model import GeneTable
+
+        try:
+            # get filesize and unit
+            input_size = os.stat(self.genes).st_size
+            total, scale, unit = ProgressReader.scale_size(input_size)
+            task = self.progress.add_task("Loading genes", total=total, unit=unit, precision=".1f")
+            # load gene table
+            self.info("Loading", "genes table from file", repr(self.genes))
+            with ProgressReader(open(self.genes, "rb"), self.progress, task, scale) as genes_file:
+                yield from GeneTable.load(io.TextIOWrapper(genes_file)).to_genes()
+        except OSError as err:
+            self.error("Fail to parse genes coordinates: {}", err)
+            raise CommandExit(err.errno) from err
+
     def _load_features(self) -> "FeatureTable":
         from ...model import FeatureTable
 
@@ -190,28 +222,45 @@ class Train(Command):  # noqa: D101
         self.success("Loaded", "a total of", len(features), "features", level=1)
         return features
 
-    def _convert_to_genes(self, features: "FeatureTable") -> List["Gene"]:
-        self.info("Converting", "features to genes")
+    def _annotate_genes(self, genes: List["Gene"], features: "FeatureTable") -> List["Gene"]:
+        from ...model import Domain
 
-        gene_count = len(set(features.protein_id))
-        unit = "gene" if gene_count == 1 else "genes"
-        task = self.progress.add_task("Converting features", total=gene_count, unit=unit, precision="")
+        # index genes by protein_id
+        gene_index = { gene.protein.id:gene for gene in genes }
+        if len(gene_index) < len(genes):
+            raise ValueError("Duplicate gene names in input genes")
 
-        genes = list(self.progress.track(
-            features.to_genes(),
-            total=gene_count,
-            task_id=task
-        ))
+        # add domains from the feature table
+        unit = "row" if len(features) == 1 else "rows"
+        task = self.progress.add_task("Annotating genes", total=len(features), unit=unit, precision="")
+        for row in self.progress.track(features, total=len(features), task_id=task):
+            # get gene by ID and check consistency
+            gene = gene_index[row.protein_id]
+            if gene.source.id != row.sequence_id:
+                raise ValueError(f"Mismatched source sequence for {row.protein_id!r}: {gene.source.id!r} != {row.sequence_id!r}")
+            elif gene.end - gene.start != row.end - row.start:
+                raise ValueError(f"Mismatched gene length for {row.protein_id!r}: {gene.end - gene.start!r} != {row.end - row.start!r}")
+            elif gene.start != row.start:
+                raise ValueError(f"Mismatched gene start for {row.protein_id!r}: {gene.start!r} != {row.start!r}")
+            elif gene.end != row.end:
+                raise ValueError(f"Mismatched gene end for {row.protein_id!r}: {gene.end!r} != {row.end!r}")
+            elif gene.strand.sign != row.strand:
+                raise ValueError(f"Mismatched gene strand for {row.protein_id!r}: {gene.strand.sign!r} != {row.strand!r}")
+            elif gene.source.id != row.sequence_id:
+                raise ValueError(f"Mismatched sequence ID {row.protein_id!r}: {gene.source.id!r} != {row.sequence_id!r}")
+            # add the row domain to the gene
+            domain = Domain(
+                name=row.domain,
+                start=row.domain_start,
+                end=row.domain_end,
+                hmm=row.hmm,
+                i_evalue=row.i_evalue,
+                pvalue=row.pvalue,
+            )
+            gene.protein.domains.append(domain)
 
-        # filter domains out
-        Annotate._filter_domains(self, genes)
-
-        self.info("Sorting", "genes by genomic coordinates")
-        genes.sort(key=operator.attrgetter("source.id", "start", "end"))
-        self.info("Sorting", "domains by protein coordinates")
-        for gene in genes:
-            gene.protein.domains.sort(key=operator.attrgetter("start", "end"))
-        return genes
+        # return
+        return list(gene_index.values())
 
     def _fit_model(self, genes: List["Gene"]) -> "ClusterCRF":
         from ...crf import ClusterCRF
@@ -219,11 +268,13 @@ class Train(Command):  # noqa: D101
         self.info("Creating", f"the CRF in [bold blue]{self.feature_type}[/] mode", level=1)
         self.info("Using", f"hyperparameters C1={self.c1}, C2={self.c2}", level=1)
         if self.select is not None:
-            self.info("Using", f"Fisher Exact Test significance threshold of {self.select}", level=1)
+            self.info("Selecting", f"features with Fisher Exact Test threshold of {self.select}", level=1)
+        self.info("Iterating", f"over features with a sliding window of size {self.window_size} and step {self.window_step}", level=1)
         crf = ClusterCRF(
             self.feature_type,
             algorithm="lbfgs",
-            overlap=self.overlap,
+            window_size=self.window_size,
+            window_step=self.window_step,
             c1=self.c1,
             c2=self.c2,
         )
@@ -302,13 +353,9 @@ class Train(Command):  # noqa: D101
                     cluster_row.start <= gene.start and gene.end <= cluster_row.end
                     for cluster_row in cluster_by_seq[seq_id]
                 ):
-                    gene = gene.with_protein(
-                        gene.protein.with_domains([d.with_probability(1) for d in gene.protein.domains])
-                    )
+                    gene = gene.with_probability(1)
                 else:
-                    gene = gene.with_protein(
-                        gene.protein.with_domains([d.with_probability(0) for d in gene.protein.domains])
-                    )
+                    gene = gene.with_probability(0)
                 labelled_genes.append(gene)
                 self.progress.update(task_id=task, advance=1)
 
@@ -377,9 +424,16 @@ class Train(Command):  # noqa: D101
             # attempt to create the output directory
             self._make_output_directory()
             # load features
+            genes = list(self._load_genes())
             features = self._load_features()
-            genes = self._convert_to_genes(features)
-            del features
+            genes = self._annotate_genes(genes, features)
+            # Sort genes
+            self.info("Sorting", "genes by coordinates", level=2)
+            genes.sort(key=operator.attrgetter("source.id", "start", "end"))
+            for gene in genes:
+                gene.protein.domains.sort(key=operator.attrgetter("start", "end"))
+            # filter domains by p-value and/or e-value
+            genes = Annotate._filter_domains(self, genes)
             # load clusters and label genes inside clusters
             clusters = self._load_clusters()
             genes = self._label_genes(genes, clusters)
